/*
 * wlanmodel/message.cc
 *
 * Copyright (C) 2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "message.h"


/** Print raw data in hex to stdout */
void
dump_bytes(const char *data, int len)
{
    int i;

    printf("(%d bytes) data: ", len);
    for(i = 1; i <= len; i++)
    {
        printf("%02hhx ", *(data + (i-1)));
        if (i % 16 == 0)
            printf("\n                 ");
    }
    printf("\n");
}

/** Print content of wlan_msg to debug output */
void
print_wlan_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    struct wlan_msg *msg = (struct wlan_msg *) message;
    int frame_size = msg->pkt_length;
    printf("(%4i bytes) ----------- WLAN MESSAGE ----------------\n", \
        frame_size + WLAN_PREAMBLE_SIZE);
    printf("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    printf("(%4i bytes) | type:\t\t%i\n", sizeof(header->type), header->type);
    printf("(%4i bytes) | id:\t\t%s\n", sizeof(header->id), header->id);
    printf("(%4i bytes) pkg_length:\t\t%i\n", sizeof(uint16_t), frame_size);
    struct ether_header *ether_hdr = (struct ether_header *) msg->wlan_pkt;
    printf("(%4i bytes) raw_packet ------------------------------\n", frame_size);
    printf("(%4i bytes) | ether_header --------------------------\n", sizeof(struct ether_header));
    printf("(%4i bytes) | | ether_dhost\t%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", \
        ETH_ALEN, ether_hdr->ether_dhost[0], ether_hdr->ether_dhost[1], \
        ether_hdr->ether_dhost[2], ether_hdr->ether_dhost[3], \
        ether_hdr->ether_dhost[4], ether_hdr->ether_dhost[5]);
    printf("(%4i bytes) | | ether_shost\t%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", \
        ETH_ALEN, ether_hdr->ether_shost[0], ether_hdr->ether_shost[1], \
        ether_hdr->ether_shost[2], ether_hdr->ether_shost[3], \
        ether_hdr->ether_shost[4], ether_hdr->ether_shost[5]);
    u_int16_t ether_type = ntohs(ether_hdr->ether_type);
    switch (ether_type)
    {
        case ETHERTYPE_IP:
            printf("(%4i bytes) | | type\t\t%i (ETHERTYPE_IP)\n", sizeof(u_int16_t), ETHERTYPE_IP);
            break;
        case ETHERTYPE_ARP:
            printf("(%4i bytes) | | type\t\t%i (ETHERTYPE_ARP)\n", sizeof(u_int16_t), ETHERTYPE_ARP);
            break;
        default:
            printf("(%4i bytes) | | type\t\t%i (unknown)\n", sizeof(u_int16_t), ether_type);
            break;
    }
    printf("------------------------------------------------------\n");
}

/** Print content of config_msg to debug output */
void
print_config_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    printf("(%4i bytes) ----------- CONFIGURE MESSAGE -----------\n", CONFIG_MSG_SIZE);
    printf("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    printf("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    printf("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct config_msg *conf_msg = (struct config_msg *) message;
    switch (conf_msg->property)
    {
        case SIOCSIWFREQ:
            printf("(%4i bytes) property:\t%4x (SIOCSIWFREQ)\n", sizeof(conf_msg->property), conf_msg->property);
            printf("(%4i bytes) value:\t%.3e\n", sizeof(conf_msg->value), conf_msg->value.freq);
            break;
    }
    printf("------------------------------------------------------\n");
}

/** Print content of ack_msg to debug output */
void
print_reg_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    printf("(%4i bytes) ----------- REGISTER MESSAGE ------------\n", \
        REGISTER_MSG_SIZE);
    printf("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    printf("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    printf("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct register_msg *reg_msg = (struct register_msg *) message;
    printf("(%4i bytes) msg_id:\t%i\n", sizeof(reg_msg->msg_id), reg_msg->msg_id);
    printf("(%4i bytes) name:\t%s\n", sizeof(reg_msg->name), reg_msg->name);
    printf("(%4i bytes) ip:\t\t%s\n", sizeof(reg_msg->ip), inet_ntoa(reg_msg->ip));
    printf("(%4i bytes) port:\t%i\n", sizeof(reg_msg->port), ntohs(reg_msg->port));
    printf("(%4i bytes) model_vif_info --------------------------\n", sizeof(model_vif_info));
    printf("(%4i bytes) | vif_id:\t%s\n", sizeof(host_id), reg_msg->vif_info.vif_id);
    mac_addr *mac = (mac_addr *) reg_msg->vif_info.vif_mac;
    printf("(%4i bytes) | vif_mac:\t%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n", \
        ETH_ALEN, ((unsigned char *) mac)[0], ((unsigned char *) mac)[1], \
        ((unsigned char *) mac)[2], ((unsigned char *) mac)[3], \
        ((unsigned char *) mac)[4], ((unsigned char *) mac)[5]);
    printf("(%4i bytes) | vif_name:\t%s\n", IFNAMSIZ + 1, reg_msg->vif_info.vif_name);
    printf("(%4i bytes) | freq:\t%.3e\n", sizeof(double), reg_msg->vif_info.freq);
    printf("------------------------------------------------------\n");
}

/** Print content of ack_msg to debug output */
void
print_ack_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    printf("(%4i bytes) ----------- ACKNOWLEDGE MESSAGE ---------\n", \
        ACK_MSG_SIZE);
    printf("(%4i bytes) msg_header ------------------------------\n", sizeof(struct msg_header));
    printf("(%4i bytes) | type:\t%i\n", sizeof(header->type), header->type);
    printf("(%4i bytes) | id:\t%s\n", sizeof(header->id), header->id);
    struct ack_msg *msg = (struct ack_msg *) message;
    printf("(%4i bytes) msg_id:\t%i\n", sizeof(uint8_t), msg->msg_id);
    printf("------------------------------------------------------\n");
}

/** Print content of a message to debug output */
void
debug_msg(const char *message)
{
    struct msg_header *header = (struct msg_header *) message;
    switch (header->type)
    {
        case WLAN_MSG:
            print_wlan_msg(message);
            break;
        case CONFIG_MSG:
            print_config_msg(message);
            break;
        case REGISTER_MSG:
            print_reg_msg(message);
            break;
        case ACK_MSG:
            print_ack_msg(message);
            break;
        default:
            printf("Warning: unknown message\n");
            return;
    }
}

/** Do nothing but imitate the printf() signature */
int
ignore_dbg(const char* text, ...) {
    return 0;
}
